/*
	Bluetooth Configuration Object:
	
	state:					integer,
	device:					string,
	profile:					string
	
	Bluetooth Status Object:
	
	activities:				[integer],
	connected:				[{
		device:					string,
		profile:					string 
	}],
*/

var bluetoothTriggers = (function() {
	var that = {};
	
	var Foundations = IMPORTS.foundations;
	
	var PalmCall = Foundations.Comms.PalmCall;
	
	var Future = Foundations.Control.Future;
	
//
	
	var initExtension = function(future, config) {
		future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
			'id': "com.palm.app.bluetooth", 'service': "com.palm.btmonitor/monitor", 
			'method': "getradiostate", 'params': {}})); 
		
		future.then(this, function(future) {
			if(future.result.radio == "on") {
				future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
					'id': "com.palm.app.bluetooth", 'service': "com.palm.bluetooth/prof", 
					'method': "profgetstate", 'params': {'profile': "all"}})); 
				
				future.then(this, function(future) {
					var profiles = future.result.profiles;
					
					for(var i = 0; i < profiles.length; i++) {
						var devices = future.result[profiles[i]];
						
						for(var j = 0; j < devices.length; j++) {
							if((devices[j].name != undefined) && (devices[j].state == "connected")) {
								config.connected.push({'device': devices[j].name.toLowerCase(), 'profile': profiles[i]});
							}
						}
					}
					
					future.result = true;
				});
			}
			else
				future.result = true;
		});
	}
	
	var addActivities = function(future, config) {
		var newProfActivity = {
			"start" : true,
			"replace": true,
			"activity": {
				"name": "bluetoothTriggerProf",
				"description" : "Bluetooth Connections Notifier",
				"type": {"foreground": true, "persist": false},
				"trigger" : {
					"method" : "palm://com.palm.bluetooth/prof/subscribenotifications",
					"params" : {'subscribe': true}
				},
				"callback" : {
					"method" : "palm://org.webosinternals.modeswitcher.srv/trigger",
					"params" : {"extension": "bluetooth"}
				}
			}
		};

		var newGapActivity = {
			"start" : true,
			"replace": true,
			"activity": {
				"name": "bluetoothTriggerGap",
				"description" : "Bluetooth Connections Notifier",
				"type": {"foreground": true, "persist": false},
				"trigger" : {
					"method" : "palm://com.palm.bluetooth/gap/subscribenotifications",
					"params" : {'subscribe': true}
				},
				"callback" : {
					"method" : "palm://org.webosinternals.modeswitcher.srv/trigger",
					"params" : {"extension": "bluetooth"}
				}
			}
		};
		
		future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
			'id': "com.palm.activitymanager", 'service': "com.palm.activitymanager", 
			'method': "create", 'params': newProfActivity})); 
		
		future.then(this, function(future) {
			config.activities.push(future.result.activityId);

			future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
				'id': "com.palm.activitymanager", 'service': "com.palm.activitymanager", 
				'method': "create", 'params': newGapActivity})); 

			future.then(this, function(future) {
				config.activities.push(future.result.activityId);
			
				future.result = true;
			});
		});
	};
	
	var delActivities = function(future, config) {
		var oldProfActivity = {
			"activityId": config.activities[0]
		};

		var oldGapActivity = {
			"activityId": config.activities[1]
		};
		
		future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
			'id': "com.palm.activitymanager", 'service': "com.palm.activitymanager", 
			'method': "cancel", 'params': oldProfActivity}));
		
		future.then(this, function(future) {
			future.nest(PalmCall.call("palm://org.webosinternals.modeswitcher.sys/", "systemCall", {
				'id': "com.palm.activitymanager", 'service': "com.palm.activitymanager", 
				'method': "cancel", 'params': oldGapActivity}));

			future.then(this, function(future) {
				config.activities = [];
			
				future.result = true;
			});
		});
	};
	
//
	
	var checkState = function(config, trigger) {
		for(var i = 0; i < config.connected.length; i++) {
			if((trigger.state == 0) && ((trigger.profile == "any") || 
				(trigger.profile == config.connected[i].profile)))
			{
				return true;
			}
			else if((trigger.state == 1) && ((trigger.profile != "any") && 
				(trigger.profile == config.connected[i].profile)))
			{
				return false;
			}
			else if((trigger.state == 2) && ((trigger.profile == "any") || 
				(trigger.profile == config.connected[i].profile)) && 
				(trigger.device.toLowerCase() == config.connected[i].device))
			{
				return true;
			}
			else if((trigger.state == 3) && ((trigger.profile != "any") && 
				(trigger.profile == config.connected[i].profile)) &&
				(trigger.device.toLowerCase() == config.connected[i].device))
			{
				return false;
			}
		}
		
		if((trigger.state == 0) || (trigger.state == 2))
			return false;
		
		return true;
	};
	
	var triggerState = function(config, trigger, args) {
		if((args.$activity) && (args.$activity.trigger) && 
			(args.$activity.trigger.notification != undefined))
		{
			var device = "unknown";
			var profile = "any";
			
			if(args.$activity.trigger.name)
				device = args.$activity.trigger.name.toLowerCase();
			
			if(args.$activity.trigger.profile)
				profile = args.$activity.trigger.profile;
			
			var index = -1;
			
			for(var i = 0; i < config.connected.length; i++) {
				if((config.connected[i].device == device) &&
					(config.connected[i].profile == profile))
				{
					index = i;
					break;
				}
			}
			
			if(((index == -1) && (!args.$activity.trigger.error) &&
				(args.$activity.trigger.notification == "notifnconnected")) || 
				((index != -1) && (((args.$activity.trigger.error) &&
				(args.$activity.trigger.notification == "notifnconnected")) || 
				(args.$activity.trigger.notification == "notifndisconnected") || 
				(args.$activity.trigger.notification == "notifndisconnecting"))))
			{
				if((trigger.state == 0) && ((trigger.profile == "any") || 
					(trigger.profile == profile)))
				{
					return true;
				}
				else if((trigger.state == 1) && ((trigger.profile == "any") || 
					(trigger.profile == profile)))
				{
					return true;
				}				
				else if((trigger.state == 2) && ((trigger.profile == "any") || 
					(trigger.profile == profile)) && (trigger.device.toLowerCase() == device))
				{
					return true;
				}
				else if((trigger.state == 3) && (((trigger.profile == "any") || 
					(trigger.profile == profile)) && (trigger.device.toLowerCase() == device)))
				{
					return true;
				}
			}
		}
		
		return false;
	};
	
// Asynchronous public functions
	
	that.initialize = function(config, triggers) {
		config.activities = [];
		config.connected = [];
		
		var future = new Future();
		
		if(triggers.length == 0)
			future.result = { returnValue: true };
		else {
			future.now(this, function(future) { 
				initExtension(future, config);
			});
			
			future.then(this, function(future) {
				future.now(this, function(future) { 
					addActivities(future, config);
				});
				
				future.then(this, function(future) {
					future.result = { returnValue: true };
				});
			});
		}
		
		return future;
	};
	
	that.shutdown = function(config) {
		config.connected = [];
		
		var future = new Future();
		
		if(config.activities.length == 0)
			future.result = { returnValue: true };
		else {
			future.now(this, function(future) { 
				delActivities(future, config);
			});
			
			future.then(this, function(future) {
				future.result = { returnValue: true };
			});
		}
		
		return future;
	};
	
//
	
	that.reload = function(config, triggers, args) {
		config.activities = [];
		
		var future = new Future();
		
		if((triggers.length == 0) || 
			(!args.$activity) || (!args.$activity.trigger) || 
			(args.$activity.trigger.returnValue == false))
		{
			future.result = { returnValue: true };
		}
		else {
			var device = "unknown";
			var profile = "any";
			
			if(args.$activity.trigger.name != undefined)
				device = args.$activity.trigger.name.toLowerCase();
			
			if(args.$activity.trigger.profile != undefined)
				profile = args.$activity.trigger.profile;
			
			if((!args.$activity.trigger.error) &&
				(args.$activity.trigger.notification == "notifnconnected"))
			{
				var index = -1;
				
				for(var i = 0; i < config.connected.length; i++) {
					if((config.connected[i].device == device) &&
						(config.connected[i].profile == profile))
					{
						index = i;
						break;
					}
				}
				
				if(index == -1)
					config.connected.push({'device': device, 'profile': profile});
			}
			else if((((args.$activity.trigger.error) &&
				(args.$activity.trigger.notification == "notifnconnected")) || 
				(args.$activity.trigger.notification == "notifndisconnected") || 
				(args.$activity.trigger.notification == "notifndisconnecting")))
			{
					if((config.connected[i].device == device) &&
						(config.connected[i].profile == profile))
					{
						config.connected.splice(i--, 1);
						break;
					}
			}
			
			future.now(this, function(future) { 
				addActivities(future, config);
			});
			
			future.then(this, function(future) {
				future.result = { returnValue: true };
			});
		}
		
		return future;
	};
	
// Synchronous public functions
	
	that.check = function(config, trigger) {
		return checkState(config, trigger);
	};
	
	that.trigger = function(config, trigger, args) {
		return triggerState(config, trigger, args);
	};
	
	return that;
}());
